# Package name
PACKAGE_NAME = "corc"

# Argument group defaults
ANSIBLE = "ANSIBLE"

# Cluster
CLUSTER = "CLUSTER"

COMPUTE = "COMPUTE"
JOB = "JOB"
NODE = "NODE"
OCI = "OCI"
RUN = "RUN"
STORAGE = "STORAGE"
SUBNET = "SUBNET"
S3 = "S3"
VCN = "VCN"

# Storage credentials secret name
STORAGE_CREDENTIALS_NAME = "storage-credentials"

# Kubernetes defaults
KUBERNETES_NAMESPACE = "default"

# Job defaults
JOB_DEFAULT_NAME = "job"
